import yaml
from litefeel.pycommon.io import read_file
from var_dump import var_dump
from typing import Any


def get_value(key: str, map: dict[str, str], default: Any) -> Any:
    if map is not None:
        return map.get(key, default)
    return None


def get_value_bool(key, map, default) -> bool:
    if map is not None:
        value = map.get(key, default)
        return value in ("true", "True", True)

    return False


def copy_bool(key, map, obj, default=False):
    value = get_value_bool(key, map, default)
    setattr(obj, key, value)


def copy_value(key, map, obj, default=None):
    value = get_value(key, map, default)
    setattr(obj, key, value)


def copy_subconfig(key: str, map: Any, subconfig: Any) -> None:
    value = get_value(key, map, None)
    if value is not None:
        subconfig.load(value)


class PushConfig:
    def __init__(self):
        self.localdir = "."
        self.remotedir = "/sdcard"
        self.recursion = False
        self.dontpush = False
        self.paths = []

    def load(self, obj):
        copy_value("localdir", obj, self, ".")
        copy_value("remotedir", obj, self, "/sdcard")
        copy_value("paths", obj, self, [])
        copy_bool("recursion", obj, self, False)
        copy_bool("dontpush", obj, self, False)


class ApkConfig:
    def __init__(self):
        self.apkpath = None
        self.run = False

    def load(self, obj):
        copy_value("apkpath", obj, self, None)
        copy_bool("run", obj, self, False)


class InstallConfig:
    def __init__(self):
        self.apkpath = None
        self.run = False
        self.devices = None

    def load(self, obj):
        copy_value("devices", obj, self, None)
        copy_value("apkpath", obj, self, None)
        copy_bool("run", obj, self, False)


class Config:
    def __init__(self):
        self.push = PushConfig()
        self.apk = ApkConfig()
        self.install = InstallConfig()

    def load(self, obj: Any) -> None:
        copy_subconfig("push", obj, self.push)
        copy_subconfig("apk", obj, self.apk)
        copy_subconfig("install", obj, self.install)

    def load_config(self, filename: str) -> None:
        data = read_file(filename)
        obj = yaml.load(data, Loader=yaml.BaseLoader)
        # var_dump(obj)
        self.load(obj)



import argparse

from ..cmd import call, getAdb
from ..config import Config

_IGNORE_PREFIXS = ("List of devices attached", "* daemon")


class Device:
    """docstring for Device"""

    __slots__ = ("serial", "online", "product", "model", "device", "raw")

    def __init__(self, line: str):
        self.raw = line
        arr = line.split()
        self.serial = arr[0]
        self.online = arr[1] == "device"
        self.product = arr[2].replace("product:", "")
        self.model = arr[3].replace("model:", "")
        self.device = arr[4].replace("device:", "")


def listOneItem(arr, index):
    if 0 < index <= len(arr):
        return arr[index - 1]
    return None


def get_devices() -> list[Device]:
    output, isOk = call('"%s" devices -l' % getAdb())
    devices: list[Device] = []
    if isOk:
        output = output.replace("\r\n", "\n").strip()
        lines = output.split("\n")
        if len(lines) > 1:
            # skip first line "List of devices attached"
            for line in lines:
                if not line.startswith(_IGNORE_PREFIXS):
                    devices.append(Device(line))
    devices.sort(key=lambda x: x.serial.lower())
    return devices


def getDevicesBySerial(devices:list[Device], serial):
    serial = serial.lower()
    return list(
        filter(lambda device: device.serial.lower().startswith(serial), devices)
    )


# return
#   List: matched devices
def filterDevices(devices: list[Device], args) -> list[Device]:
    if len(devices) == 0:
        print("No devices connected")
        return []

    if args is None:
        if len(devices) == 1:
            return devices[:]
        print("devices count:%d  please set devices command" % len(devices))
        return devices[:]

    selects: list[Device] = []
    for arg in args:
        device = None
        if len(arg) == 1:
            if arg == "a":
                return devices[:]
            device = listOneItem(devices, int(arg))
        elif len(arg) >= 2:
            tmp = getDevicesBySerial(devices, arg)
            if len(tmp) == 1:
                device = tmp[0]
            elif len(tmp) > 1:
                print("serial prefix %s is not unique" % arg)
                return []
        if device is not None:
            selects.append(device)
    return selects


def printDevices(devices: list[Device]):
    if devices is None:
        return
    for i in range(len(devices)):
        print("%-3d %s" % (i + 1, devices[i].raw))


##### for other script
def getSerials(devices: list[Device]) -> list[str]:
    serials = []
    for d in devices:
        serials.append(d.serial)
    return serials


def addArgumentParser(parser):
    parser.add_argument(
        "-d",
        "--devices",
        nargs="*",
        help="""filter of devices, [a | n | serial]
            a: all devices
            n: index of devices list(start with 1)
            serial: devices serial (at least 2 char)
            not argument is show device list""",
    )


def doArgumentParser(args) -> tuple[list[str], list[Device]]:
    devices = get_devices()
    if args.devices is not None and len(args.devices) == 0:
        printDevices(devices)
        return ([], [])

    devices = filterDevices(devices, args.devices)
    serials = getSerials(devices)
    return (serials, devices)


def docommand(args: argparse.Namespace, cfg: Config) -> None:
    if args.list:
        printDevices(get_devices())
        exit(0)

    devices = filterDevices(get_devices(), args.devices)
    printDevices(devices)


##### end for other script
# -------------- main ----------------
def addcommand(parser: argparse.ArgumentParser) -> None:
    parser.add_argument(
        "-d",
        "--devices",
        nargs="+",
        help="filter of devices, [n | serial | a] n:index of list(start with 1), serial:at least 2 char, a:all",
    )
    parser.add_argument("-l", "--list", action="store_true", help="show devices list")



import argparse
import os
from argparse import _SubParsersAction
from typing import Any

from litefeel.pycommon.io import read_file

from .config import Config
from .subcommands import (
    adbdevice,
    adbpush,
    apkinfo,
    apkinstall,
    apkuninstall,
    apksigner,
    assetbundleinfo,
    il2cpp,
    asshader
)

_VERSION_FILE_NAME = "version.txt"


def get_version() -> str:
    dir_of_this_script = os.path.split(__file__)[0]
    version_file_path = os.path.join(dir_of_this_script, _VERSION_FILE_NAME)
    return read_file(version_file_path).strip()


class Command:
    def __init__(self, name: str, command: Any, help: str):
        self.name = name
        self.command = command
        self.help = help


def addsubcommands(subparser: argparse._SubParsersAction, commands: list[Command]) -> None:
    for cmd in commands:
        parser = subparser.add_parser(cmd.name, help=cmd.help)
        parser.set_defaults(docommand=cmd.command.docommand)
        cmd.command.addcommand(parser)


def add_global_params(parser: argparse.ArgumentParser) -> None:
    parser.add_argument("-c", "--config", dest="config", help="global config")
    parser.add_argument(
        "--version", action="version", version=f"%(prog)s {get_version()}"
    )


def main(_args=None):
    parser = argparse.ArgumentParser(
        usage="%(prog)s [options]", description="show android device list"
    )

    add_global_params(parser)

    commands = [
        Command("devices", adbdevice, "show android device list"),
        Command("push", adbpush, "push files to android device"),
        Command("install", apkinstall, "install apk file"),
        Command("uninstall", apkuninstall, "uninstall apk file"),
        Command("apk", apkinfo, "show apk packageName/activityName"),
        Command("sign", apksigner, "sign apk with android debug(only windows)"),
        Command("ab", assetbundleinfo, "extract unity asset bundle information"),
        Command("il2cpp", il2cpp, "extract unity il2cpp information"),
        Command("asshader", asshader, "simplify asset studio shader preview data"),
    ]

    subparser = parser.add_subparsers(title="sub commands", dest="subcommand")
    addsubcommands(subparser, commands)

    args = parser.parse_args(_args)
    if args.subcommand is None:
        parser.print_help()
        exit(0)

    cfg = Config()
    if args.config is not None:
        cfg.load_config(args.config)
    args.docommand(args, cfg)


# -------------- main ----------------
if __name__ == "__main__":
    main()



import argparse
import os
from typing import Optional

from ..cmd import call, getAdb
from ..config import Config
from . import adbdevice, apkinfo

# BASE_DIR="F:/release"
BASE_DIR = ""


def getApks(path, filters):
    apks = os.listdir(path)
    apks = filter(lambda filename: filename.endswith(".apk"), apks)
    if filters is not None:

        def myfilterfun(filename):
            for f in filters:
                if f not in filename:
                    return False
            return True

        apks = filter(myfilterfun, apks)
    apks = map(lambda filename: os.path.join(path, filename), apks)
    return list(apks)


def getNewst(apks: list[str]) -> Optional[str]:
    if len(apks) == 0:
        return None
    apks = sorted(apks, key=os.path.getmtime, reverse=True)
    return apks[0]


def filterApks(fileorpath: str, filters) -> Optional[str]:
    apk = fileorpath

    if os.path.isdir(fileorpath):
        apks = getApks(fileorpath, filters)
        if len(apks) == 0:
            print("can not found apk file in %s " % fileorpath)
            exit(1)
        apk = getNewst(apks)
    return apk


def install(apks: list[str], serials: list[str], run: bool) -> None:
    adb = getAdb()
    last = len(apks) - 1
    for i in range(0, len(apks)):
        apk = apks[i]
        isrun = run and last == i
        for serial in serials:
            cmd = '"%s" -s %s install -r "%s"' % (adb, serial, apk)
            _, isOk = call(cmd, True)
            print(isOk)
            if isOk and isrun:
                apkinfo.run(apk, [serial])


def docommand(args: argparse.Namespace, cfg: Config) -> None:
    serials, devices = adbdevice.doArgumentParser(args)
    if not serials:
        exit(0)

    path = args.apkpath or cfg.install.apkpath or "."
    path = os.path.abspath(os.path.join(BASE_DIR, path))

    apks = filterApks(path, args.filter)

    if serials is not None and apks is not None:
        install([apks], serials, args.run or cfg.install.run)


def addcommand(parser: argparse.ArgumentParser) -> None:
    parser.add_argument("-f", "--filter", nargs="*", help="filtered by file name")
    parser.add_argument(
        "-r", "--run", action="store_true", help="run app after install"
    )
    parser.add_argument("apkpath", nargs="?")
    adbdevice.addArgumentParser(parser)



import argparse
import os
from zipfile import ZipFile

from genericpath import isdir, isfile
from litefeel.pycommon.io import makedirs, write_file

from ..cmd import call
from ..config import Config
from ..errors import raise_error


def extract(file, output_folder):
    zipfile = ZipFile(file, 'r')
    write_file(os.path.join(output_folder, "libil2cpp.so"), zipfile.read("lib/arm64-v8a/libil2cpp.so"))
    write_file(os.path.join(output_folder, "global-metadata.dat"), zipfile.read("assets/bin/Data/Managed/Metadata/global-metadata.dat"))
    zipfile.close()

def do_file(apkfile, output_folder):
    if not os.path.isfile(apkfile):
        raise_error(f'file not fond:{apkfile}')

    extract(apkfile, output_folder)
    il2cppso = os.path.join(output_folder, "libil2cpp.so")
    metadatadat = os.path.join(output_folder, "global-metadata.dat")
    cmd = f'Il2CppDumper.exe "{il2cppso}" "{metadatadat}" "{output_folder}"'
    call(cmd)

def docommand(args: argparse.Namespace, cfg: Config) -> None:
    output = args.output
    apkpath = args.apkpath
    if os.path.isfile(apkpath):
        if output is None:
            output = os.path.splitext(apkpath)[0]
        makedirs(output)
        do_file(apkpath, output)
    else:
        raise_error(f"apkpath is not exits:{apkpath}")


def addcommand(parser: argparse.ArgumentParser) -> None:
    parser.add_argument("-o", "--output", help="output file or folder")
    parser.add_argument("apkpath", help="apk file")



import argparse
from os import error
import os.path
import re

from ..cmd import call, getAapt, getAdb
from ..config import Config
from ..errors import raise_error
from . import adbdevice


def firstitem(arr):
    return arr[0] if len(arr) > 0 else None


def parse(apk):
    aapt = getAapt()
    cmd = '"%s" dump badging "%s"' % (aapt, apk)
    output, isOk = call(cmd)
    if isOk:
        packagename = firstitem(re.findall(r"package: name='(\S+?)'", output))
        activityname = firstitem(
            re.findall(r"launchable-activity: name='(\S+?)'", output)
        )
        return (packagename, activityname, output)
    return (None, None, None)


def stop(apk, serials):
    packagename, _, _ = parse(apk)
    adb = getAdb()
    for serial in serials:
        cmd = '"%s" -s %s shell am force-stop "%s"' % (adb, serial, packagename)
        call(cmd)


def run(apk, serials):
    packagename, activityname, _ = parse(apk)
    adb = getAdb()
    for serial in serials:
        cmd = '"%s" -s %s shell am start -S "%s/%s"' % (
            adb,
            serial,
            packagename,
            activityname,
        )
        call(cmd)


def docommand(args: argparse.Namespace, cfg: Config) -> None:
    if args.apkpath is not None:
        cfg.apk.apkpath = args.apkpath

    if cfg.apk.apkpath is None:
        raise_error("Missing parameter: apkpath")
        return

    apkpath = os.path.abspath(cfg.apk.apkpath)
    if not os.path.isfile(apkpath):
        raise_error("apkpath is not file")
        return

    if args.run:
        serials, _ = adbdevice.doArgumentParser(args)
        run(apkpath, serials)
    elif args.stop:
        serials, _ = adbdevice.doArgumentParser(args)
        stop(apkpath, serials)
    else:
        packagename, activityname, output = parse(apkpath)
        print(output)


def addcommand(parser: argparse.ArgumentParser) -> None:
    group = parser.add_mutually_exclusive_group()
    group.add_argument("-r", "--run", action="store_true", help="run app")
    group.add_argument("-s", "--stop", action="store_true", help="run app")
    parser.add_argument("apkpath", nargs="?")
    adbdevice.addArgumentParser(parser)



import os
import shlex
import subprocess
import sys

from semantic_version import Version

from .errors import raise_error


# return (output, isOk)
def call(cmd: str, printOutput: bool = False) -> tuple[str, bool]:
    # print(f"{printOutput = }, {cmd = }")
    if sys.platform == "win32":
        args = cmd
    else:
        # linux must split arguments
        args = shlex.split(cmd)
    try:
        if printOutput:
            isOk = subprocess.call(args) == 0
            return "", isOk

        data = subprocess.check_output(args)
        # python3 output is bytes
        output = data.decode("utf-8")
        return output, True
    except subprocess.CalledProcessError as callerr:
        print(f"cmd = {cmd}, callerr.output = {callerr.output}", file=sys.stderr)
        return (callerr.output, False)
    except IOError as ioerr:
        print(f"cmd = {cmd}, ioerr = {ioerr}", file=sys.stderr)
        return "", False


def getAdb() -> str:
    androidHome = os.getenv("ANDROID_HOME")
    if androidHome is None:
        androidHome = os.getenv("ANDROID_SDK")
    if androidHome is None:
        androidHome = os.getenv("ANDROID_SDK_ROOT")
    if androidHome is None:
        # print('can not found ANDROID_HOME/ANDROID_SDK in environment value')
        return "adb"

    return os.path.join(androidHome, "platform-tools/adb")


def getAapt() -> str:
    androidHome = os.getenv("ANDROID_HOME")
    if androidHome is None:
        androidHome = os.getenv("ANDROID_SDK")
    if androidHome is None:
        androidHome = os.getenv("ANDROID_SDK_ROOT")
    if androidHome is None:
        print("can not found ANDROID_HOME/ANDROID_SDK in environment value")
        return "aapt"

    aaptname = "aapt.exe" if sys.platform == "win32" else "aapt"

    buildtools = os.path.join(androidHome, "build-tools")
    if os.path.isdir(buildtools):
        vers = list(map(lambda v: Version(v), os.listdir(buildtools)))
        vers.sort(reverse=True)
        for ver in vers:
            filename = os.path.join(buildtools, str(ver), aaptname)
            if os.path.isfile(filename):
                return filename

    raise_error("can not found aapt in ANDROID_HOME/ANDROID_SDK")
    return ""

def getApksigner() -> str:
    androidHome = os.getenv("ANDROID_HOME")
    if androidHome is None:
        androidHome = os.getenv("ANDROID_SDK")
    if androidHome is None:
        androidHome = os.getenv("ANDROID_SDK_ROOT")
    if androidHome is None:
        print("can not found ANDROID_HOME/ANDROID_SDK in environment value")
        return "aapt"

    # aaptname = "apksigner.bat" if sys.platform == "win32" else "apksigner.bat"
    aaptname = "apksigner.bat"

    buildtools = os.path.join(androidHome, "build-tools")
    if os.path.isdir(buildtools):
        vers = list(map(lambda v: Version(v), os.listdir(buildtools)))
        vers.sort(reverse=True)
        for ver in vers:
            filename = os.path.join(buildtools, str(ver), aaptname)
            if os.path.isfile(filename):
                return filename

    raise_error("can not found aapt in ANDROID_HOME/ANDROID_SDK")
    return ""

def get_unity_editor_dir(editor_dir):
    def is_editor_dir(dir):
        if dir is None:
            return False
        if os.path.isdir(dir):
            if os.path.isfile(os.path.join(dir, 'Unity.exe')):
                return True
        return False
    if is_editor_dir(editor_dir):
        return editor_dir
    editor_dir = os.getenv("UNITY_EDITOR_ROOT")
    if is_editor_dir(editor_dir):
        return editor_dir
    raise_error('can not found unity editor')

def get_unity_binary2text(unity_editor_dir):
    return os.path.join(get_unity_editor_dir(unity_editor_dir), 'Data/Tools/binary2text.exe')

def get_unity_webextract(unity_editor_dir):
    return os.path.join(get_unity_editor_dir(unity_editor_dir), 'Data/Tools/WebExtract.exe')


import argparse
import os
from typing import Optional

from ..cmd import call, getApksigner
from ..config import Config
from . import adbdevice, apkinfo

# BASE_DIR="F:/release"
BASE_DIR = ""


def getApks(path, filters):
    apks = os.listdir(path)
    apks = filter(lambda filename: filename.endswith(".apk"), apks)
    if filters is not None:

        def myfilterfun(filename):
            for f in filters:
                if f not in filename:
                    return False
            return True

        apks = filter(myfilterfun, apks)
    apks = map(lambda filename: os.path.join(path, filename), apks)
    return list(apks)


def getNewst(apks: list[str]) -> Optional[str]:
    if len(apks) == 0:
        return None
    apks = sorted(apks, key=os.path.getmtime, reverse=True)
    return apks[0]


def filterApks(fileorpath: str, filters) -> Optional[str]:
    apk = fileorpath

    if os.path.isdir(fileorpath):
        apks = getApks(fileorpath, filters)
        if len(apks) == 0:
            print("can not found apk file in %s " % fileorpath)
            exit(1)
        apk = getNewst(apks)
    return apk


def sign(apks: list[str]) -> None:
    adb = getApksigner()
    last = len(apks) - 1

    defaultkeystor = os.path.expanduser("~/.android/debug.keystore")
    print(defaultkeystor)
    # defaultkeystor = r"C:\Users\Administrator\.android\debug.keystore"
    storepass = "android"
    for i in range(0, len(apks)):
        apk = apks[i]
        cmd = '"%s" sign --ks "%s" --ks-pass pass:android "%s"' % (adb, defaultkeystor, apk)
        _, isOk = call(cmd, True)
        print(isOk)


def docommand(args: argparse.Namespace, cfg: Config) -> None:
    path = args.apkpath or cfg.install.apkpath or "."
    path = os.path.abspath(os.path.join(BASE_DIR, path))

    apks = filterApks(path, args.filter)

    if apks is not None:
        sign([apks])


def addcommand(parser: argparse.ArgumentParser) -> None:
    parser.add_argument("-f", "--filter", nargs="*", help="filtered by file name")
    # parser.add_argument(
    #     "-r", "--run", action="store_true", help="run app after install"
    # )
    parser.add_argument("apkpath", nargs="?")
    adbdevice.addArgumentParser(parser)



import argparse
import os
from enum import IntEnum

from litefeel.pycommon.io import makedirs, read_lines, write_lines

from ..config import Config
from ..errors import raise_error


class SimplifyType(IntEnum):
    Format = 1
    RemoveContent = 2


def _lstrips(lines: list[str]) -> None:
    for i in range(len(lines)):
        lines[i] = lines[i].lstrip()


def _format(lines: list[str]) -> None:
    space = ""
    for i in range(len(lines)):
        line = lines[i]

        prev_space = space
        n = 0
        if "{" in line:
            n += 1
        if "}" in line:
            n -= 1

        if n > 0:
            space += "\t"
        elif n < 0:
            space = space[:-1]

        if "{" in line:
            lines[i] = prev_space + line
        else:
            lines[i] = space + line


def _remove_shader_content(lines: list[str]) -> list[str]:
    skipCurLine = False
    hasSubProgram = False
    output_lines = []
    for line in lines:
        if hasSubProgram:
            if line == '""':
                continue
            if not line.startswith("Keywords {"):
                if '"' in line:
                    skipCurLine = not skipCurLine
                    continue
                if skipCurLine:
                    continue
                else:
                    hasSubProgram = False
        if line.startswith("SubProgram "):
            hasSubProgram = True
        output_lines.append(line)
    return output_lines


def do_file(file: str, output_file: str, simply: SimplifyType) -> None:
    lines = read_lines(file)
    _lstrips(lines)

    if simply == SimplifyType.RemoveContent:
        lines = _remove_shader_content(lines)
    _format(lines)
    write_lines(output_file, lines)


def docommand(args: argparse.Namespace, cfg: Config) -> None:
    output = args.output
    input = args.shader
    if os.path.isfile(input):
        if output is None:
            output = input
        makedirs(output, isfile=True)
        do_file(input, output, SimplifyType(args.simplify))
    else:
        raise_error(f"shader is not exits:{input}")


def addcommand(parser: argparse.ArgumentParser) -> None:
    parser.add_argument(
        "-s",
        "--simplify",
        nargs="?",
        choices=[SimplifyType.Format.value, SimplifyType.RemoveContent.value],
        default=1,
        type=int,
        help="simplify file 1:format 2:remove shader content",
    )
    parser.add_argument("-o", "--output", help="output file or folder")
    parser.add_argument("shader", help="asset stuido shader preview file")



import argparse
from genericpath import isdir, isfile
import os
from posixpath import abspath
import re
import tempfile
import shutil
from litefeel.pycommon.io import makedirs

from ..cmd import call, get_unity_binary2text, get_unity_webextract
from ..config import Config
from ..errors import raise_error
from . import adbdevice

INFO_EXT = ".txt"


def bine2text(file, unity_editor_dir):
    binary2text = get_unity_binary2text(unity_editor_dir)
    cmd = f'"{binary2text}" "{file}"'
    output, isOk = call(cmd)
    if isOk:
        return file +".txt"

def extract(file, unity_editor_dir):
    webextract = get_unity_webextract(unity_editor_dir)
    cmd = f'"{webextract}" "{file}"'
    output, isOk = call(cmd)
    if isOk:
        for root, dirs, files in os.walk(file+"_data"):
            for f in files:
                if not f.endswith('.resS'):
                    return os.path.join(root, f)
                    
def do_file(input_file, output_file, unity_editor_dir):
    if not os.path.isfile(input_file):
        raise_error(f'file not fond:{input_file}')

    with tempfile.TemporaryDirectory() as tmpdirname:
        tmpunityfile = os.path.join(tmpdirname, 'tmp.unity3d')
        shutil.copy(input_file, tmpunityfile)
        outfile = extract(tmpunityfile, unity_editor_dir)
        assert outfile
        outfile = bine2text(outfile, unity_editor_dir)
        assert outfile
        makedirs(output_file, True)
        shutil.copy(outfile, output_file)

def do_folder(input_dir:str, output_dir:str, ext:str, unity_editor_dir:str) -> None:
    for root, dir, files in os.walk(input_dir):
        for f in files:
            if ext and not f.endswith(ext):
                continue
            input_file = os.path.join(root, f)
            rel_name = os.path.relpath(input_file, input_dir)
            output_file = os.path.join(output_dir, rel_name + INFO_EXT)
            do_file(input_file, output_file, unity_editor_dir)


def docommand(args: argparse.Namespace, cfg: Config) -> None:
    output = args.output
    abpath = args.abpath
    unity_editor_dir = args.unityeditordir
    if os.path.isfile(abpath):
        if output is None:
            output = abpath + INFO_EXT
        elif os.path.isdir(output):
            output = os.path.join(output, os.path.basename(abpath) + INFO_EXT)
        do_file(abpath, output, unity_editor_dir)
    elif os.path.isdir(abpath):
        if output is None:
            output = abpath
        elif os.path.isfile(output):
            raise_error("output cannot be file when abpath is folder")
        do_folder(abpath, output, args.ext, unity_editor_dir)
    else:
        raise_error(f"abpath is not exits:{abpath}")


def addcommand(parser: argparse.ArgumentParser) -> None:
    parser.add_argument("-o", "--output", help="output file or folder")
    parser.add_argument("-e", "--ext", nargs='?', const='', default=".unity3d", help="assetbundle extions name")
    parser.add_argument('-u', '--unityeditordir',nargs='?', help='unity editor folder')
    parser.add_argument("abpath", help="assetbundle file or folder")
    adbdevice.addArgumentParser(parser)



import argparse
import hashlib
import json
import os
from typing import Callable

from litefeel.pycommon.io import read_file, write_file

from ..cmd import call, getAdb
from ..config import Config, PushConfig
from . import adbdevice

date_dict: dict[str, str] = {}
g_serial = ""
hashfunc: Callable[[str], str]
errors = []

push_cfg: PushConfig


def file_sha1(file: str) -> str:
    sha1 = hashlib.sha1()
    with open(file, mode="rb") as f:
        while True:
            data = f.read(65536)
            if not data:
                break
            sha1.update(data)

    return sha1.hexdigest()


def file_mtime(file: str) -> str:
    return str(os.path.getmtime(push_cfg.localdir))


def load_json(filename: str) -> dict:
    if filename is None:
        return {}
    if os.path.exists(filename):
        return json.loads(read_file(filename))
    return {}


def pull(file: str, prefixLocal: str, prefixRemote: str) -> bool:
    file = file.replace("\\", "/")
    local = file
    remote = file
    if file.startswith(prefixLocal):
        remote = prefixRemote + file[len(prefixLocal) :]

    _, isOk = call(
        '"%s" -s %s pull "%s" "%s"' % (getAdb(), g_serial, remote, local), True
    )
    return isOk


def push(file: str, prefixLocal: str, prefixRemote: str, dontpush: bool) -> None:
    file = file.replace("\\", "/")
    local = file
    remote = file
    relname: str = file
    if file.startswith(prefixLocal):
        remote = prefixRemote + file[len(prefixLocal) :]
        relname = file[len(prefixLocal) :]

    oldhash = date_dict.get(relname, "")
    nowhash = hashfunc(local)

    if oldhash != nowhash or dontpush:
        isOk = True
        if not dontpush:
            rellocal = os.path.relpath(local, ".")
            _, isOk = call(
                '"%s" -s %s push "%s" "%s"' % (getAdb(), g_serial, rellocal, remote), True
            )
        if isOk:
            date_dict[relname] = nowhash
        else:
            errors.append(relname)


def filePush(path, prefixLocal, prefixRemote, dontpush):
    files = os.listdir(path)
    for f in files:
        file = "%s/%s" % (path, f)
        if os.path.isfile(file):
            push(file, prefixLocal, prefixRemote, dontpush)


def walkPush(path, prefixLocal, prefixRemote, dontpush):
    for root, _, files in os.walk(path):
        for f in files:
            push("%s/%s" % (root, f), prefixLocal, prefixRemote, dontpush)


def push_all(cfg: PushConfig, serial: str, hashjson: str) -> None:
    global g_serial, date_dict
    g_serial = serial
    errors.clear()

    local = cfg.localdir
    remote = cfg.remotedir

    if hashjson is not None:
        if not pull(hashjson, local, remote):
            if os.path.isfile(hashjson):
                os.remove(hashjson)

    # print(hashjsonfile)
    date_dict = load_json(hashjson)

    for path in cfg.paths:
        path = os.path.abspath(path)
        if os.path.isfile(path):
            push(path, local, remote, cfg.dontpush)
        elif os.path.isdir(path):
            if cfg.recursion:
                walkPush(path, local, remote, cfg.dontpush)
            else:
                filePush(path, local, remote, cfg.dontpush)
        else:
            print("%s: No such file or directory" % path)

    if hashjson is not None:
        write_file(hashjson, json.dumps(date_dict, sort_keys=True, indent=4))
        if not cfg.dontpush:
            push(hashjson, local, remote, cfg.dontpush)

    if len(errors) > 0:
        print("error paths:")
        map(print, errors)


def _is_relpath(path, root):
    if not path.startswith(root):
        return False
    if len(path) == len(root):
        return True
    return path[len(root)] == "/"


def _normal_path(p, root):
    if os.path.isabs(p):
        p = p.replace("\\", "/")
    else:
        p = os.path.abspath(p).replace("\\", "/")
        if not _is_relpath(p, root):
            p = os.path.join(root, p)
    p = os.path.abspath(p).replace("\\", "/")
    return p if os.path.exists(p) and _is_relpath(p, root) else None


def docommand(args: argparse.Namespace, cfg: Config) -> None:
    serials, devices = adbdevice.doArgumentParser(args)
    if not serials:
        exit(0)

    global push_cfg

    push_cfg = cfg.push

    if args.dontpush:
        push_cfg.dontpush = True

    if args.localdir is not None:
        push_cfg.localdir = args.localdir
    push_cfg.localdir = os.path.abspath(push_cfg.localdir).replace("\\", "/")
    if args.remotedir is not None:
        push_cfg.remotedir = args.remotedir

    if args.recursion:
        push_cfg.recursion = True

    paths = args.path[:] or push_cfg.paths or ["."]
    paths = [_normal_path(p, push_cfg.localdir) for p in paths]
    push_cfg.paths = [p for p in paths if p]

    global hashfunc
    hashfunc = file_mtime if args.hash == "mtime" else file_sha1

    for device in devices:
        hashjson = "%s_%s.json" % (device.model, device.serial)
        hashjson = os.path.abspath(hashjson)
        push_all(push_cfg, device.serial, hashjson)


def addcommand(parser: argparse.ArgumentParser) -> None:
    parser.add_argument(
        "-r", dest="recursion", action="store_true", help="recursion all file"
    )
    parser.add_argument(
        "-n",
        dest="newst",
        action="store_true",
        help="only push new file by last modify files, see -j",
    )
    parser.add_argument(
        "-j",
        dest="hashjson",
        action="store",
        nargs="?",
        default="",
        help="hash json file, default: ./$deviceMode_$deviceSerial.json",
    )
    parser.add_argument(
        "--hash",
        dest="hash",
        nargs="?",
        default="sha1",
        choices=["sha1", "mtime"],
        help="hash function: mtime or sha1, default:mtime",
    )
    parser.add_argument(
        "--localdir",
        dest="localdir",
        help="local prefix and remote prefix, will replace local prefix to remote prefix",
    )
    parser.add_argument(
        "--remotedir",
        dest="remotedir",
        help="local prefix and remote prefix, will replace local prefix to remote prefix",
    )
    parser.add_argument(
        "--dontpush",
        dest="dontpush",
        action="store_true",
        help="only outout json file, not really push file to remote",
    )
    parser.add_argument("path", nargs="*", help="file or directory")
    adbdevice.addArgumentParser(parser)



import os

from setuptools import find_packages, setup

_DIR_OF_THIS_SCRIPT = os.path.split(__file__)[0]
_VERSION_FILE_NAME = "version.txt"
_VERSION_FILE_PATH = os.path.join(_DIR_OF_THIS_SCRIPT, "adbtool", _VERSION_FILE_NAME)
_README_FILE_NAME = "README.md"
_README_FILE_PATH = os.path.join(_DIR_OF_THIS_SCRIPT, _README_FILE_NAME)


with open(_VERSION_FILE_PATH, "r") as fh:
    version = fh.read().strip()

with open(_README_FILE_PATH, "r") as fh:
    long_description = fh.read()

setup(
    name="adbtool",
    version=version,
    keywords=["Android", "adb"],
    description="A friendly android adb command-line tool",
    long_description=long_description,
    long_description_content_type="text/markdown",
    license="MIT",
    url="https://github.com/litefeel/adbtool",
    author="litefeel",
    author_email="litefeel@gmail.com",
    packages=find_packages(exclude=["test", "tests"]),
    include_package_data=True,
    platforms="any",
    install_requires=["litefeel-pycommon", "PyYAML", "semantic_version", "var_dump"],
    scripts=[],
    entry_points={"console_scripts": ["adbt = adbtool.adbtool:main"]},
    # See https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        "Development Status :: 5 - Production/Stable",
        # Indicate who your project is intended for
        "Intended Audience :: Developers",
        "Topic :: Software Development :: Libraries :: Python Modules",
        # Pick your license as you wish (should match "license" above)
        "License :: OSI Approved :: MIT License",
        # Specify the Python versions you support here. In particular, ensure
        # that you indicate whether you support Python 2, Python 3 or both.
        "Programming Language :: Python :: 3",
    ],
    python_requires=">=3.9",
)
